from djpb.django_to_proto import django_to_proto, django_to_proto_bytes
from djpb.gen_proto import gen_proto_for_models
from djpb.proto_to_django import proto_to_django
from djpb.registry import register_model
from .custom_field import CustomField, ReadOnlyField



import typing
from collections import defaultdict

from djpb.stubs import DjModelType, ProtoMsgType

MODEL_TO_PROTO_CLS: typing.DefaultDict[
    DjModelType, typing.List[ProtoMsgType]
] = defaultdict(lambda: [])

PROTO_CLS_TO_MODEL: typing.Dict[ProtoMsgType, DjModelType] = {}


def register_model(proto_classes: typing.List[ProtoMsgType]):
    def decorator(django_model: DjModelType):
        MODEL_TO_PROTO_CLS[django_model] = proto_classes

        for proto_class in proto_classes:
            PROTO_CLS_TO_MODEL[proto_class] = django_model

        return django_model

    return decorator



from dataclasses import dataclass

from .serializers import SaveNode
from .stubs import DjModel, ProtoMsg


@dataclass
class CustomField:
    proto_type: str
    null: bool = False

    def update_proto(self, django_obj: DjModel, proto_obj: ProtoMsg, field_name: str):
        pass

    def update_django(self, node: "SaveNode", proto_obj: ProtoMsg, field_name: str):
        pass


@dataclass
class ReadOnlyField(CustomField):
    query: str = None

    def update_proto(self, django_obj, proto_obj, field_name):
        model = type(django_obj)
        value = (
            model.objects.filter(pk=django_obj.pk)
            .values_list(self.query or field_name, flat=True)
            .first()
        )
        if value is None:
            return
        setattr(proto_obj, field_name, value)
